package scenic3;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;

import org.slim3.controller.Controller;
import org.slim3.controller.Navigation;
import org.slim3.controller.upload.FileItem;
import org.slim3.util.BooleanUtil;
import org.slim3.util.DoubleUtil;
import org.slim3.util.IntegerUtil;
import org.slim3.util.LongUtil;
import org.slim3.util.StringUtil;

/**
 * Controller extends slim3's controller.
 * <p>
 * このコントローラは、slim3のコントローラに幾つかの機能を追加し、
 * Scenic3で自動生成されるコントローラの基底クラスとなります。
 * </p>
 * @author shuji.w6e
 * @since 0.1.0
 */
public abstract class ScenicController extends Controller {
    /** */
    public static final ScenicController STATIC_CONTROLLER = new ScenicController() {

        @Override
        protected Navigation run() throws Exception {
            return null;
        }

        @Override
        public String toString() {
            return "STATIC_CONTROLLER";
        }

        @Override
        public String getActionMethodName() {
            return null;
        }

        @Override
        public ScenicPage getPage() {
            return null;
        }
    };

    // set when creating instance.
    protected String path;
    // set when creating instance.
    protected UrlPattern urlPattern;
    // request variables
    private Map<String, String> vars = null;

    /*
     * (non-Javadoc)
     * @see org.slim3.controller.Controller#setUp()
     */
    @Override
    protected Navigation setUp() {
        //        this.setupPage(page);
        return getPage().setUp();
    }

    /*
     * (non-Javadoc)
     * @see org.slim3.controller.Controller#tearDown()
     */
    @Override
    protected void tearDown() {
        getPage().tearDown();
    }

    /**
     * リクエストパスに含まれる埋め込み変数を抽出して返します。
     * 例えば、/aaa/{name1}/{name2} というリクエストパスが定義されており、
     * /aaa/123/456
     * name1 でこのメソッドを呼び出すことにより、「123」が取得できます。
     * @param name パス変数の名前
     * @return 対応する値、存在しない場合はnull
     */
    protected String var(String name) {
        if (vars == null) {
            vars = new HashMap<String, String>();
            Matcher m = urlPattern.pattern.matcher(this.path);
            if (!m.matches()) throw new IllegalStateException("missmatch");
            int count = urlPattern.vars.length;
            for (int i = 0; i < count; i++) {
                vars.put(urlPattern.vars[i], m.group(i + 1));
            }
        }
        return vars.get(name);
    }

    protected String[] vars(String name) {
        return var(name).split("/");
    }

    protected int varAsInteger(String name) {
        return Integer.valueOf(var(name));
    }

    protected Integer[] varsAsInteger(String name) {
        String[] vars = var(name).split("/");
        Integer[] result = new Integer[vars.length];
        for (int i = 0; i < vars.length; i++) {
            result[i] = Integer.valueOf(vars[i]);
        }
        return result;
    }

    protected int[] varsAsInt(String name) {
        String[] vars = var(name).split("/");
        int[] result = new int[vars.length];
        for (int i = 0; i < vars.length; i++) {
            result[i] = Integer.valueOf(vars[i]);
        }
        return result;
    }

    protected long varAsLong(String name) {
        return Long.valueOf(var(name));
    }

    protected Long[] varsAsLong(String name) {
        String[] vars = var(name).split("/");
        Long[] result = new Long[vars.length];
        for (int i = 0; i < vars.length; i++) {
            result[i] = Long.valueOf(vars[i]);
        }
        return result;
    }

    protected long[] varsAsPLong(String name) {
        String[] vars = var(name).split("/");
        long[] result = new long[vars.length];
        for (int i = 0; i < vars.length; i++) {
            result[i] = Long.valueOf(vars[i]);
        }
        return result;
    }

    protected FileItem asFileItem(String name) {
        return requestScope(name);
    }

    protected void requestScope(CharSequence name, Object value) throws NullPointerException {
        super.requestScope(name, value);
    }

    protected <T> T requestScope(CharSequence name) throws NullPointerException {
        return super.<T> requestScope(name);
    }

    protected <T> T removeRequestScope(CharSequence name) throws NullPointerException {
        return super.<T> removeRequestScope(name);
    }

    @Override
    protected String[] paramValues(CharSequence name) throws NullPointerException {
        return super.paramValues(name);
    }

    @Override
    protected String param(CharSequence name) throws NullPointerException {
        return super.param(name);
    }

    protected void sessionScope(CharSequence name, Object value) throws NullPointerException {
        super.sessionScope(name, value);
    }

    protected <T> T sessionScope(CharSequence name) throws NullPointerException {
        return super.<T> sessionScope(name);
    }

    protected <T> T removeSessionScope(CharSequence name) throws NullPointerException {
        return super.<T> removeSessionScope(name);
    }

    @Override
    protected boolean isPost() {
        return super.isPost();
    }

    @Override
    protected boolean isGet() {
        return super.isGet();
    }

    @Override
    protected boolean isPut() {
        return super.isPut();
    }

    @Override
    protected boolean isDelete() {
        return super.isDelete();
    }

    @Override
    protected Navigation forward(String path) {
        return super.forward(path);
    }

    @Override
    protected Navigation redirect(String path) {
        return super.redirect(path);
    }

    @Override
    protected void show(String fileName, byte[] data) {
        super.show(fileName, data);
    }

    @Override
    protected void show(String fileName, InputStream in) {
        super.show(fileName, in);
    }

    @Override
    protected void download(String fileName, byte[] data) {
        super.show(fileName, data);
    }

    @Override
    protected void download(String fileName, InputStream in) {
        super.download(fileName, in);
    }

    public abstract ScenicPage getPage();

    public abstract String getActionMethodName();

    protected void setupPage(ScenicPage page) {
        if (page == null) throw new IllegalArgumentException("page == null");
        page.setController(this);
        page.servletContext = this.servletContext;
        page.request = this.request;
        page.response = this.response;
        page.basePath = this.basePath;
        page.errors = this.errors;
    }

    /**
     * get request parameter as String
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter value
     * @since 0.5.0
     */
    protected String paramAsString(String name, String defultValue) {
        if (request.getParameter(name) == null) return StringUtil.toString(defultValue);
        return param(name);
    }

    /**
     * get request parameters as String
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter values
     * @since 0.5.0
     */
    protected String[] paramsAsString(String name, String[] defultValue) {
        if (request.getParameter(name) == null) return defultValue;
        return paramValues(name);
    }
    
    /**
     * get request parameter as Integer
     * <p>convert String value to Integer using {@link IntegerUtil}.</p>
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter value
     * @since 0.5.0
     */
    protected Integer paramAsInteger(String name, String defultValue) {
        if (request.getParameter(name) == null) return IntegerUtil.toInteger(defultValue);
        return IntegerUtil.toInteger(param(name));
    }
    
    /**
     * get request parameter as int
     * <p>convert String value to int using {@link IntegerUtil}.</p>
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter value
     * @since 0.5.0
     */
    protected int paramAsPrimitiveInt(String name, String defultValue) {
        if (request.getParameter(name) == null) return IntegerUtil.toPrimitiveInt(defultValue);
        return IntegerUtil.toPrimitiveInt(param(name));
    }
    
    /**
     * get request parameter as Long
     * <p>convert String value to Long using {@link LongUtil}.</p>
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter value
     * @since 0.5.0
     */
    protected Long paramAsLong(String name, String defultValue) {
        if (request.getParameter(name) == null) return LongUtil.toLong(defultValue);
        return LongUtil.toLong(param(name));
    }

    /**
     * get request parameter as long
     * <p>convert String value to long using {@link LongUtil}.</p>
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter value
     * @since 0.5.0
     */
    protected long paramAsPrimitiveLong(String name, String defultValue) {
        if (request.getParameter(name) == null) return LongUtil.toPrimitiveLong(defultValue);
        return LongUtil.toPrimitiveLong(param(name));
    }

    /**
     * get request parameter as Boolean
     * <p>convert String value to Boolean using {@link BooleanUtil}.</p>
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter value
     * @since 0.5.0
     */
    protected Boolean paramAsBoolean(String name, String defultValue) {
        if (request.getParameter(name) == null) return BooleanUtil.toBoolean(defultValue);
        return BooleanUtil.toBoolean(param(name));
    }

    /**
     * get request parameter as boolean
     * <p>convert String value to boolean using {@link BooleanUtil}.</p>
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter value
     * @since 0.5.0
     */
    protected boolean paramAsPrimitiveBoolean(String name, String defultValue) {
        if (request.getParameter(name) == null) return BooleanUtil.toPrimitiveBoolean(defultValue);
        return BooleanUtil.toPrimitiveBoolean(param(name));
    }

    /**
     * get request parameter as Double
     * <p>convert String value to Double using {@link DoubleUtil}.</p>
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter value
     * @since 0.5.0
     */
    protected Double paramAsDouble(String name, String defultValue) {
        if (request.getParameter(name) == null) return DoubleUtil.toDouble(defultValue);
        return DoubleUtil.toDouble(param(name));
    }

    /**
     * get request parameter as double
     * <p>convert String value to double using {@link DoubleUtil}.</p>
     * @param name parameter name
     * @param defultValue parameter as String value if not exist
     * @return parameter value
     * @since 0.5.0
     */
    protected double paramAsPrimitiveDouble(String name, String defultValue) {
        if (request.getParameter(name) == null) return DoubleUtil.toPrimitiveDouble(defultValue);
        return DoubleUtil.toPrimitiveDouble(param(name));
    }
}
