package scenic3;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slim3.util.ClassUtil;

/**
 * 
 * @author shuji.w6e
 * @since 0.1.0
 */
public abstract class UrlMatcherImpl implements UrlMatcher {

    private final String pageClassName;
    private final String pagePath;
    private final List<UrlPattern> patterns;
    private final Map<UrlPattern, String> controllers;
    
    /**
     * Constructor.
     * @param pageClassName
     * @param pagePath
     */
    public UrlMatcherImpl(String pageClassName, String pagePath) {
        if (pageClassName == null) throw new IllegalArgumentException("pageClassName == null");
        if (pagePath == null) throw new IllegalArgumentException("path == null");
        this.pageClassName = pageClassName;
        this.pagePath = pagePath;
        this.patterns = Collections.synchronizedList(new ArrayList<UrlPattern>());
        this.controllers = Collections.synchronizedMap(new HashMap<UrlPattern, String>());
    }

    protected void add(UrlPattern pattern, String contollerClass) {
        this.patterns.add(pattern);
        this.controllers.put(pattern, contollerClass);
    }

    @Override
    public ScenicController match(String path, String method) {
        if (!path.startsWith(pagePath)) return null;
        try {
            for (UrlPattern urlPattern : this.patterns) {
                if (urlPattern.matches(method) && urlPattern.pattern.matcher(path).matches()) {
                    ClassLoader loader = Thread.currentThread().getContextClassLoader();
                    Class<?> clazz = Class.forName(controllers.get(urlPattern), true, loader);
                    ScenicController c = ClassUtil.newInstance(clazz);
                    c.path = path;
                    c.urlPattern = urlPattern;
                    return c;
                }
            }
        } catch (Throwable e) {
            throw new UncompleteAnnotationException("Controller class creation error.", e);
        }
        // matched pattern not found.
        return null;
    }
    
    /*
     * (non-Javadoc)
     * @see scenic3.UrlMatcher#getPageClassName()
     */
    @Override
    public final String getPageClassName() {
        return pageClassName;
    }

    @Override
    public String toString() {
        StringBuilder str = new StringBuilder();
        for (UrlPattern pattern : patterns) {
            str.append(pattern).append(", ").append(controllers.get(pattern)).append("\n");
        }
        return str.toString();
    }

}
