package scenic3;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import scenic3.util.PathUtil;

/**
 * 
 * @author shuji.w6e
 * @since 0.1.0
 */
public class UrlPattern {

    final Pattern pattern;
    final boolean defaultPath;
    final String path;
    final String[] vars;
    final String method;

    public UrlPattern(String pagePath, String actionPath) {
        this(pagePath, actionPath, null);
    }

    public UrlPattern(String pagePath, String actionPath, String method) {
        this(pagePath, actionPath, method, false);
    }
    
    UrlPattern(String pagePath, String actionPath, String method, boolean enableIndexUrl) {
        this.defaultPath = actionPath.length() == 0;
        this.method = method;
        StringBuilder pathBuf = new StringBuilder();
        pathBuf.append(PathUtil.appendEndSlashIfNeed(pagePath));
        if (defaultPath || actionPath.charAt(0) != '/') {
            pathBuf.append(actionPath);
        } else { // /で始まるケース
            pathBuf.append(actionPath.substring(1));
        }
        this.path = pathBuf.toString();
        StringBuilder reg = new StringBuilder("^");
        if (!pagePath.equals("/")) {
            reg.append(PathUtil.appendStartSlashIfNeed(pagePath));
        }
        if (reg.charAt(reg.length() - 1) == '/') {
            reg.setLength(reg.length() - 1);
        }
        if (actionPath.length() == 0) {
            if (enableIndexUrl) {
                reg.append("/.*");
            } else {
                reg.append("/");
            }
        } else {
            String p = actionPath.replace(".", "\\.").replaceAll("\\{([^\\/]+)\\}", "([^\\\\/]+)");
            p = p.replace("/([^\\/]+)*", "/(.*)");
            reg.append(PathUtil.appendStartSlashIfNeed(p));
            reg.append('$');
        }
        this.pattern = Pattern.compile(reg.toString());
        if (actionPath.indexOf('{') == -1) {
            vars = new String[0];
        } else {
            Matcher matcher = pattern.matcher(this.path);
            matcher.matches();
            int count = matcher.groupCount();
            vars = new String[count];
            for (int i = 0; i < count; i++) {
                String param = matcher.group(i + 1);
                if (param.charAt(param.length() - 1) == '*') {
                    vars[i] = param.substring(1, param.length() - 2);
                } else {
                    vars[i] = param.substring(1, param.length() - 1);
                }
            }
        }
    }

    @Override
    public String toString() {
        return pattern.pattern();
    }

    public boolean matches(String targetMethod) {
        if (method == null) return true;
        return this.method.equalsIgnoreCase(targetMethod);
    }

}
