package scenic3;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;


/**
 * 
 * @author shuji.w6e
 * @since 0.1.0
 */
public class UrlsImpl implements Urls {

    protected final List<UrlMatcher> matchers;

    public UrlsImpl() {
        matchers = Collections.synchronizedList(new LinkedList<UrlMatcher>());
    }

    /**
     * UrlMacherを指定して追加する
     * @param matcher
     */
    protected void add(UrlMatcher matcher) {
        if (matcher == null) throw new IllegalArgumentException("matcher == null");
        matchers.add(matcher);
    }

    protected void excludes(String... patterns) {
        add(new StaticUrlMatcher(patterns));
    }

    
    @Override
    @Deprecated
    public ScenicController createController(String requestPath) {
        return createController(requestPath, "GET");
    }
    
    /**
     * RequestPathを指定して、処理を行うPageAdapterを取得する
     * @return
     */
    public ScenicController createController(String path, String method) {
        for (UrlMatcher matcher : matchers) {
            ScenicController c = matcher.match(trim(path), method);
            if (c != null) return c;
        }
        return null;
    }

    private String trim(String path) {
        int paramSeparatorIndex = path.indexOf('?');
        if (paramSeparatorIndex == -1) {
            return path;
        } else {
            return path.substring(0, paramSeparatorIndex);
        }
    }
    
    /**
     * 
     * @param matcherClass
     * @return
     * @since 0.5.0
     */
    public boolean isSupport(Class<? extends ScenicPage> pageClass){
        String pageClassName = pageClass.getName();
        for (UrlMatcher matcher : this.matchers) {
            if (matcher == UrlMatcher.EXCLUDE_MATCHER) continue;
            if (matcher instanceof StaticUrlMatcher) continue;
            if (pageClassName.equals(matcher.getPageClassName())) return true;
        }
        return false;
    }

    @Override
    public String toString() {
        StringBuilder str = new StringBuilder();
        for (UrlMatcher matcher : matchers) {
            str.append(matcher.toString()).append("\n");
        }
        return str.toString();
    }

}
