package scenic3.annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * リクエストパラメータに付与するアノテーション。
 * <p>
 * アクションメソッドにこのアノテーションを付与する事で、
 * アクションメソッドの呼び出し時にHttpRequestからパラメータを取得し、
 * そのパラメータをメソッドの引数に渡すことができる。
 * </p>
 * <p>
 * 次のアクションメソッドの宣言は、リクエストパラメータからidを取得し、Long値に変換する
 * <pre><code>
 * @ActionPath("find")
 * public Navigation find(@RequestParam("id") Long id)
 * </code></pre>
 * すなわち
 * <pre><code>
 * @ActionPath("find")
 * public Navigation find() {
 *   Long id = super.asLong("id");
 * </code></pre>
 * と同等となる。
 * <p>
 * 
 * @author shuji.w6e
 * @since 0.1.0
 */
@Retention(RetentionPolicy.CLASS)
@Target(ElementType.PARAMETER)
@Inherited
public @interface RequestParam {
    /** パラメータ名 */
    String value();
    /** パラメータのデフォルト値として利用される値（文字列） */
    String[] defaultValue() default {};
}
