package scenic3.apt;


/**
 * Argument of an action method.
 * @author shuji.w6e
 * @since 0.1.0
 */
public class Argument {
    
    /** a type of this argument */
    public final String type;
    /** a name of this argument */
    public final String name;
    /** a http parameter name of this argument */
    public final String paramName;
    /** a path parameter of this argument */
    public final String varName;
    /** default values of this argument. only the 1st element is used in non-array type. */
    public final String[] defaultValues;

    Argument(String type, String name, String paramName, String varName, String defaultValue) {
        this.type = type;
        this.name = name;
        this.paramName = paramName;
        this.varName = varName;
        this.defaultValues = new String[]{defaultValue};
    }

    Argument(String type, String name, String paramName, String varName, String[] defaultValues) {
        this.type = type;
        this.name = name;
        this.paramName = paramName;
        this.varName = varName;
        this.defaultValues = defaultValues;
    }

    public static Argument simpleInstance(String type, String name) {
        return new Argument(type, name, null, null, (String) null);
    }
    
    /**
     * create instance with HTTP parameter argument default value
     * @param type
     * @param name
     * @param paramName
     * @param defaultValue
     * @return
     * @since 0.5.0
     */
    public static Argument paramInstance(String type, String name, String paramName, String defaultValue) {
        return new Argument(type, name, paramName, null, defaultValue);
    }

    public static Argument paramsInstance(String type, String name, String paramName, String[] defaultValues) {
        return new Argument(type, name, paramName, null, defaultValues);
    }

    public static Argument paramInstance(String type, String name, String paramName) {
        return new Argument(type, name, paramName, null, (String) null);
    }
    
    public static Argument varInstance(String type, String name, String varName) {
        return new Argument(type, name, null, varName, (String) null);
    }
}
