package scenic3.apt;

import org.slim3.util.StringUtil;
        
/**
 * method creator.
 * @author shuji.w6e
 * @since 0.2.0
 */
class MethodCreater {

    static String make(String actionMethodName, Argument... arguments) throws UnsupportedArgumentException {
        StringBuilder method = new StringBuilder();
        method.append(actionMethodName);
        method.append('(');
        for (Argument arg : arguments) {
            if (arg.type.equals("javax.servlet.http.HttpServletRequest")) {
                method.append("super.request, ");
            } else if (arg.type.equals("javax.servlet.http.HttpServletResponse")) {
                method.append("super.response, ");
            } else if (arg.paramName != null) {
                if (arg.type.equals("java.lang.String")) {
                    method.append("paramAsString(\"" + arg.paramName + "\", " + defaultValue(arg) + "), ");
                } else if (arg.type.equals("java.lang.String[]")) {
                    method.append("paramsAsString(\"" + arg.paramName + "\", " + defaultValues(arg) + "), ");
                } else if (arg.type.equals("int")) {
                    method.append("paramAsPrimitiveInt(\"" + arg.paramName + "\", " + defaultValue(arg) + "), ");
                } else if (arg.type.equals("java.lang.Integer")) {
                    method.append("paramAsInteger(\"" + arg.paramName + "\", " + defaultValue(arg) + "), ");
                } else if (arg.type.equals("long")) {
                    method.append("paramAsPrimitiveLong(\"" + arg.paramName + "\", " + defaultValue(arg) + "), ");
                } else if (arg.type.equals("java.lang.Long")) {
                    method.append("paramAsLong(\"" + arg.paramName + "\", " + defaultValue(arg) + "), ");
                } else if (arg.type.equals("boolean")) {
                    method.append("paramAsPrimitiveBoolean(\"" + arg.paramName + "\", " + defaultValue(arg) + "), ");
                } else if (arg.type.equals("java.lang.Boolean")) {
                    method.append("paramAsBoolean(\"" + arg.paramName + "\", " + defaultValue(arg) + "), ");
                } else if (arg.type.equals("double")) {
                    method.append("paramAsPrimitiveDouble(\"" + arg.paramName + "\", " + defaultValue(arg) + "), ");
                } else if (arg.type.equals("java.lang.Double")) {
                    method.append("paramAsDouble(\"" + arg.paramName + "\", " + defaultValue(arg) + "), ");
                } else if (arg.type.equals("org.slim3.controller.upload.FileItem")) {
                    method.append("super.asFileItem(\"" + arg.paramName + "\"), ");
                } else {
                    throw new UnsupportedArgumentException(arg);
                }
            } else if (arg.varName != null) {
                if (arg.type.equals("java.lang.String")) {
                    method.append("super.var(\"" + arg.varName + "\"), ");
                } else if (arg.type.equals("java.lang.String[]")) {
                    method.append("super.vars(\"" + arg.varName + "\"), ");
                } else if (arg.type.equals("int") || arg.type.equals("java.lang.Integer")) {
                    method.append("super.varAsInteger(\"" + arg.varName + "\"), ");
                } else if (arg.type.equals("int[]")) {
                    method.append("super.varsAsInt(\"" + arg.varName + "\"), ");
                } else if (arg.type.equals("java.lang.Integer[]")) {
                    method.append("super.varsAsInteger(\"" + arg.varName + "\"), ");
                } else if (arg.type.equals("long") || arg.type.equals("java.lang.Long")) {
                    method.append("super.varAsLong(\"" + arg.varName + "\"), ");
                } else if (arg.type.equals("long[]")) {
                    method.append("super.varsAsPLong(\"" + arg.varName + "\"), ");
                } else if (arg.type.equals("java.lang.Long[]")) {
                    method.append("super.varsAsLong(\"" + arg.varName + "\"), ");
                } else {
                    throw new UnsupportedArgumentException(arg);
                }
            } else {
                throw new UnsupportedArgumentException(arg);
            }
        }
        if (0 < arguments.length) method.setLength(method.length() - 2);
        method.append(')');
        return method.toString();
    }

    private static String defaultValue(Argument arg) {
        return (arg.defaultValues == null || arg.defaultValues.length == 0 || arg.defaultValues[0] == null) ?
          "null" : "\"" + arg.defaultValues[0] + "\"";
    }

    private static String defaultValues(Argument arg) {
        StringBuilder args = new StringBuilder();
        if (arg.defaultValues == null || arg.defaultValues.length == 0) {
            args.append("new String[0]");
        } else {
            args.append("new String[]{");
            for (String dv : arg.defaultValues) {
              args.append("\"" + dv + "\", ");
            }
            if (arg.defaultValues.length > 0) args.setLength(args.length() - 2);
            args.append('}');
        }

        return args.toString();
    }
}
