package scenic3.apt;

/**
 * description for a ScenicController.
 * @author shuji.w6e
 * @since 0.1.0
 */
public class ScenicControllerDesc {

    public final String pageClassName;
    public final String packageName;
    public final String className;
    public final String qName;
    public final String actionPath;
    public final String pagePath;
    public final String actionMethodName;
    public final String method;
    public final Argument[] arguments;

    public ScenicControllerDesc(String pageClassName, String rootPackage, String pagePath, String actionPath,
            String actionMethodName, String method, Argument... params) {
        this.pageClassName = pageClassName;
        this.packageName = getPackageName(rootPackage, pagePath);
        this.className = toControllerClassName(actionPath, method);
        this.qName = this.packageName + "." + className;
        this.actionPath = actionPath;
        this.pagePath = pagePath;
        this.actionMethodName = actionMethodName;
        this.method = method;
        this.arguments = params;
    }

    private String getPackageName(String rootPackage, String pagePath) {
        StringBuilder name = new StringBuilder(rootPackage);
        name.append(".controller");
        for (String path : pagePath.split("\\/")) {
            if (path.length() == 0) continue;
            name.append(".$").append(path);
        }
        return name.toString();
    }

    private String toControllerClassName(String actionPath, String method) {
        StringBuilder name = new StringBuilder();
        if (actionPath == null) {
            name.append("$Index");
        } else if (actionPath.equals("")) {
            name.append("$StrictIndex");
        } else {
            name.append("_");
            for (char c : actionPath.toCharArray()) {
                switch (c) {
                case '{':
                    break;
                case '}':
                    break;
                case '.':
                    name.append('$');
                    break;
                case '/':
                    name.append('_');
                    break;
                case '*':
                    name.append("_ASTA");
                    break;
                default:
                    name.append(c);
                }
            }
        }
        if (method != null) {
            name.append("$$").append(method);
        }
        return name.toString();
    }

}
