package scenic3.apt;

import java.io.IOException;
import java.io.PrintWriter;

/**
 * ScenicController generator.
 * @author shuji.w6e
 * @since 0.1.0
 */
public class ScenicControllerGenerator extends CodeGenerator {

    /** desc */
    final ScenicControllerDesc desc;

    public ScenicControllerGenerator(ScenicControllerDesc desc, PrintWriter writer) throws IOException {
        super(writer);
        this.desc = desc;
    }

    @Override
    public void generate() throws UnsupportedArgumentException {
        try {
            p("package %s;", desc.packageName);
            p();
            p("// Controller for %s#%s", desc.pageClassName, desc.actionMethodName);
            p("// @javax.annotation.Generated");
            p("public final class %s extends %s {", desc.className, "scenic3.ScenicController"); // TODO 固定
            p();
            p("    private final %s page;", desc.pageClassName);
            p();
            p("    public %s() {", desc.className);
            p("        this.page = new %s();", desc.pageClassName);
            p("    }");
            p();
            p("    @Override");
            p("    public final org.slim3.controller.Navigation setUp() {");
            p("        setupPage(page);");
            p("        return super.setUp();");
            p("    }");
            p();
            p("    @Override");
            p("    public final org.slim3.controller.Navigation run() throws Exception {");
            p("        return page.%s;", MethodCreater.make(desc.actionMethodName, desc.arguments));
            p("    }");
            p();
            p("    @Override");
            p("    public final %s getPage() {", desc.pageClassName);
            p("        return this.page;");
            p("    }");
            p();
            p("    @Override");
            p("    protected final org.slim3.controller.Navigation handleError(Throwable error) throws Throwable {");
            p("        return page.handleError(error);");
            p("    }");
            p();
            p("    @Override");
            p("    public final String getActionMethodName() {");
            p("        return \"%s\";", desc.actionMethodName);
            p("    }");
            p();
            if (desc.method != null) {
                p("    // Override for REST");
                p("    @Override");
                p("    protected boolean isGet() {");
                p("        return %s;", "get".equalsIgnoreCase(desc.method));
                p("    }");
                p();
                p("    // Override for REST");
                p("    @Override");
                p("    protected boolean isPost() {");
                p("        return %s;", "post".equalsIgnoreCase(desc.method));
                p("    }");
                p();
                p("    // Override for REST");
                p("    @Override");
                p("    protected boolean isPut() {");
                p("        return %s;", "put".equalsIgnoreCase(desc.method));
                p("    }");
                p();
                p("    // Override for REST");
                p("    @Override");
                p("    protected boolean isDelete() {");
                p("        return %s;", "delete".equalsIgnoreCase(desc.method));
                p("    }");
                p();
            }
            p("}");
        } finally {
            writer.flush();
            writer.close();
        }
    }
}
