package scenic3.apt;

import java.io.IOException;
import java.io.PrintWriter;

import scenic3.ActionPath;

/**
 * Generator for UrlMatcher class.
 * @author shuji.w6e
 * @since 0.1.0
 */
public class UrlMatcherGenerator extends CodeGenerator {

    final UrlMatcherDesc desc;

    public UrlMatcherGenerator(UrlMatcherDesc desc, PrintWriter writer) throws IOException {
        super(writer);
        this.desc = desc;
    }

    @Override
    public void generate() {
        try {
            p("package %s;", desc.packageName);
            p();
            p("// @javax.annotation.Generated");
            p("public class %s extends scenic3.UrlMatcherImpl {", desc.className);
            p();
            p("    private static final %s INSTANCE = new %s();", desc.className, desc.className);
            p("    /**");
            p("     * get a instance of this class.");
            p("     */");
            p("    public static %s get() {", desc.className);
            p("        return INSTANCE;");
            p("    }");
            p();
            p("    // Constructor.");
            p("    private %s() {", desc.className);
            p("        super(\"%s\", \"%s\");", desc.pageClassName, desc.pagePath);
            for (ActionPath actionPath : desc.patterns.keySet()) {
                if (actionPath.method.isEmpty()) {
                    p(
                        "        super.add(new scenic3.UrlPattern(\"%s\", \"%s\"), \"%s\");",
                        desc.pagePath,
                        actionPath.path,
                        desc.patterns.get(actionPath));
                } else {
                    p(
                            "        super.add(new scenic3.UrlPattern(\"%s\", \"%s\", \"%s\"), \"%s\");",
                            desc.pagePath,
                            actionPath.path,
                            actionPath.method,
                            desc.patterns.get(actionPath));
                    
                }
            }
            if (desc.defaultPattern != null) {
                p(
                        "        super.add(new scenic3.IndexUrlPattern(\"%s\"), \"%s\");",
                        desc.pagePath,
                        desc.defaultPattern);
            }
            p("    }");
            p();
            p();
            p("}");
        } finally {
            writer.flush();
            writer.close();
        }
    }
}
