package scenic3.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.Charset;

public class Files {

    private static final String LINE_SEPARATOR = System.getProperty("line.separator");

    public static FileInputStream newFileInputStream(File file) throws IOException {
        if (file == null) throw new IllegalArgumentException("file is null.");
        return new FileInputStream(file);
    }

    public static String readAsString(File file) throws IOException {
        return readAsString(file, Charset.defaultCharset(), LINE_SEPARATOR);
    }

    public static String readAsString(File file, String charset) throws IOException {
        return readAsString(file, Charset.forName(charset), LINE_SEPARATOR);
    }

    public static String readAsString(File file, Charset charset) throws IOException {
        return readAsString(file, charset, LINE_SEPARATOR);
    }

    public static String readAsString(File file, Charset charset, String lineSeparator) throws IOException {
        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new InputStreamReader(newFileInputStream(file), charset));
            StringBuilder text = new StringBuilder();
            for (;;) {
                String line = reader.readLine();
                if (line == null) break;
                text.append(line).append(lineSeparator);
            }
            return text.toString();
        } finally {
            Stream.close(reader);
        }
    }
}
