package scenic3.util;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Stream utility.
 * <p></p>
 * @author shuji.w6e
 * @since 0.1.0
 */
public class Stream {

    private static final int DEFAULT_BUFFER_SIZE = 1024 * 2;

    public static void copy(InputStream input, OutputStream output) throws IOException {
        copy(input, output, DEFAULT_BUFFER_SIZE);
    }

    public static void copy(InputStream input, OutputStream output, int bufSize) throws IOException {
        BufferedInputStream bin = (input instanceof BufferedInputStream) ? (BufferedInputStream) input
                : new BufferedInputStream(input);
        BufferedOutputStream bout = (output instanceof BufferedOutputStream) ? (BufferedOutputStream) output
                : new BufferedOutputStream(output);
        copy(bin, bout, bufSize);
    }

    public static void copy(BufferedInputStream input, BufferedOutputStream output) throws IOException {
        copy(input, output, DEFAULT_BUFFER_SIZE);
    }

    public static void copy(BufferedInputStream input, BufferedOutputStream out, int bufSize) throws IOException {
        byte[] buf = new byte[bufSize];
        for (;;) {
            int len = input.read(buf);
            if (len == -1) break;
            out.write(buf, 0, len);
        }
        out.flush();
    }

    public static void close(Closeable closeable) {
        if (closeable == null) return;
        try {
            closeable.close();
        } catch (IOException e) {
            // do nothing
        }
    }
}
